/**
 * ----------------------------------------------------------------------------
 * (c) 20012 by Les Retraites Populaires
 * ----------------------------------------------------------------------------
 *
 * RollbackExceptionInterceptor.java
 *
 * PROJECT: FE Migration
 * 
 *
 * $Source$
 * $Revision: 76885 $
 * ----------------------------------------------------------------------------
 * WHEN         WHO     VER     DESCRIPTION
 * 03.05.12     ADL     1.0     Creation
 * ----------------------------------------------------------------------------
 */
package ch.rp.core.shared.ejb.interceptor;

import java.util.Map;

import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;

import ch.rp.core.shared.utils.ActionIdGenerator;
import ch.rp.services.sharedcontext.SharedContextHolder;
import ch.rp.services.sharedcontext.SharedContextService;

/**
 * Intercepteur de container utilisé par JBoss pour injecter le SharedContext. 
 * Cet intercepteur est insérer avant tout autres intercepteurs du container. (Avant l'intercepteur des transactions).
 * 
 * Attention: comme mentionné dans la doc JBoss ci-dessous, la méthode getTarget() ne doit pas être utilisée dans 
 * 			  cet intercepteur.
 * 
 * 		- Semantic difference between container interceptor(s) and Java EE interceptor(s) API
 *		-
 *		-  Although the container interceptors are modeled to be similar to the Java EE interceptors, 
 *		-  there are some differences in the API semantics. One such difference is that invoking on 
 *		-  javax.interceptor.InvocationContext.getTarget() method is illegal for container interceptors 
 *		-  since these interceptors are invoked way before the EJB components are setup or instantiated.
 *				
 * voir https://docs.jboss.org/author/display/AS72/Container+interceptors pour de plus amples explications sur les intercepteur de container.
 * 
 * @author adl
 *
 */
public class SharedContextEJBServerInterceptor {
    private static final Logger LOGGER = LoggerFactory.getLogger(SharedContextEJBServerInterceptor.class);

    public static final String USERNAME_MDC_KEY = "username";
    public static final String ACTION_ID_MDC_KEY = "actionId";

    @AroundInvoke
    public Object injectSharedContext(InvocationContext ctx) throws Exception {

        String methodName = "";
        if (ctx.getMethod() != null) {
            methodName = ctx.getMethod().toString();
        }

        if (!SharedContextService.getInstance().hasThreadSharedContext()) {
            //Remote call 
            LOGGER.debug("Remote EJB Call ({})", methodName);
            try {
                Map<String, Object> contextData = ctx.getContextData();
                if (contextData != null) {
                    setupSharedContext(ctx, contextData);
                } else {
                    LOGGER.error("No context Data in EJB Call ({})", methodName);
                }
                return ctx.proceed();
            } finally {
                cleanUpSharedContext();
            }
        } else {
            //Local Call
            LOGGER.debug("Local EJB Call ({})", methodName);
            return ctx.proceed();
        }
    }

    private void setupSharedContext(InvocationContext ctx, Map<String, Object> contextData) {
        //Retrieve SharedContextHolder
        SharedContextHolder holder = (SharedContextHolder) contextData.get(SharedContextHolder.SHARED_CONTEXT_HOLDER_KEY);
        if (holder != null) {
            SharedContextService.getInstance().setThreadSharedContext(holder);
            if (holder.getActionId() == null) {
				holder.setActionId(ActionIdGenerator.generateActionId());
            }

            MDC.put(ACTION_ID_MDC_KEY, holder.getActionId());
            MDC.put(USERNAME_MDC_KEY, holder.getUserName());
        } else {
            String methodName = "";
            if (ctx.getMethod() != null) {
                methodName = ctx.getMethod().toString();
            }
            LOGGER.error("No rp shared context in EJB Call ({})", methodName);
        }
    }

    private void cleanUpSharedContext() {
        SharedContextService.getInstance().setThreadSharedContext(null);
        MDC.clear();
    }
}
