package ch.rp.services.sharedcontext;

/**
 * Service d'accès aux variables partagées transmises implicitement entre composants.
 */
public final class SharedContextService {

    /** Singleton pattern. */
    private static SharedContextService instance = new SharedContextService();

    /** Context Holder pour chaque thread. */
    private ThreadLocal<SharedContextHolder> threadContext = new ThreadLocal<SharedContextHolder>();

    /** Context Holder pour le cient (pas de threadLocal) */
    private SharedContextHolder clientContext = new SharedContextHolder();

    private boolean isClient = false;

    /**
     * Fournit le singleton du service.
     */
    public static SharedContextService getInstance() {
        return instance;
    }

    /**
     * Fournit le contexte pour le thread courant, le crée si besoin.
     * @return instance SharedContextHolder
     */
    public SharedContextHolder getThreadSharedContext() {
        SharedContextHolder result = threadContext.get();
        if (result == null) {
            result = new SharedContextHolder();
            threadContext.set(result);
        }
        return result;
    }

    /**
     * Contrôle que le SharedContext est initilisé pour le Thread courant
     * @return vrai si le thread courant a un sharedContext sinon faux
     */
    public boolean hasThreadSharedContext() {
        return threadContext.get() != null;
    }

    /**
     * Set le SharedContextHolder dans le threadLocal.
     * (Utiliser par l'intercepteur EJB pour injecter le context 
     *  récu dans l'appel)
     */
    public void setThreadSharedContext(SharedContextHolder holder) {
        threadContext.set(holder);
    }

    /**
     * Fournit le contexte pour le client courant, le crée si besoin.
     * @return instance SharedContextHolder
     */
    public SharedContextHolder getClientSharedContext() {
        isClient = true;
        return clientContext;
    }

    public boolean isClient() {
        return isClient;
    }
}
